function BaseBoard(row, col, name, mode, scale = 1) {
  if (name == null) return;
  this.name = name;
  this.piece_row = row;
  this.piece_col = col;
  this.blank_row = row;
  this.blank_col = col;

  this.boundary_top = 0;
  this.boundary_left = 0;

  this.blank_size = 50 * scale;
  this.border_width = 3 * scale;
  this.piece_font_size = 30 * scale;
  this.piece_bound_top = 0;
  this.piece_bound_left = 0;
  this.bound_font_size = 20 * scale;
  this.die_blank_size = 32 * scale;
  this.die_piece_font_size = 24 * scale;
  this.movelist_font_size = 12 * scale;

  this.div_board = this.new_div("div_board_" + name);
  this.div_piece = this.new_div("div_piece_" + name);
  this.div_board = this.new_div("div_board_" + name);
  this.div_movelist = this.new_div("div_movelist_" + name);
  this.div_piece_score = this.new_div("div_piece_score" + name);

  this.div_endgame_movelist = this.new_div("div_endgame_movelist_" + name);
  this.div_path = null;

  this.LANG_EN = 0;
  this.LANG_CH = 1;
  this.LANG = this.LANG_CH;

  this.STR_NUM = ["０", "１", "２", "３", "４", "５", "６", "７", "８", "９"];
  this.EN_NUM = ["", "ａ", "ｂ", "ｃ", "ｄ", "ｅ", "ｆ", "ｇ", "ｈ"];

  /*test*/
  this.isstopboard = false;
  this.his_create_name = false;
  this.his_openf = "";
  this.his_step = 0;  // keydown 
  this.metaChar = false;  // keydown
  this.exampleKey = 16; // keydown
  this.split_str = "\r\n"; // windows->\r\n, linux->\n
  this.os_type = "\r\n";  // windows->\r\n, linux->\n
  this.turn = 0; // 1 my turn 0 opp
  this.board_data = ""; // board information
  this.step = -1;
  this.info_row = new Array();  // user information row 
  this.info_error_row = new Array();  // user error information row
  this.info = new Array();  // user information
  this.info_error = new Array();  // user error information
  this.acttime = new Array(); // board history user time
  this.actlist = new Array(); // board history user move
  this.tmp = null;
  this.player = 0;
  this.player_name = new Array(); // 0: players 1: player1 2:player2
  this.socket = null;
  this.clr = 1; // 2: white, 1: black
  this.mode = mode; // 1: index.html, 0: history, 2: abort 3: observation 4: contest_history
  this.PCE_QUA = new Map([
    ["X", 1],
    ["O", 1],
  ]);

  this.PCE_MAP = {
    "E": { en: "", ch: "", clr: "EMPTY" },
    "X": { en: "X", ch: "黑", clr: "BLACK" },
    "O": { en: "O", ch: "白", clr: "WHITE" },
  };
  this.PCE_TYPE = {
    "BLACK": { className: "piece_black" },
    "WHITE": { className: "piece_white" },
  };
}

BaseBoard.prototype = {
  new_div: function (id) {
    document.write("<div id=\"" + id + "\"></div>");
    return document.getElementById(id);
  },
  new_div_shape: function (className, top, left, height, width, font_size, t_index, css_text = "") {
    var div = document.createElement("div");
    div.className = className;
    if (t_index != null) { div.id = t_index; }
    //if(t_index != 0){div.id = t_index;}
    div.style.cssText = "width:" + width + "px;";
    div.style.cssText += "height:" + height + "px;";
    div.style.cssText += "top:" + top + "px;";
    div.style.cssText += "left:" + left + "px;";
    div.style.cssText += "line-height:" + height + "px;";
    div.style.cssText += "font-size:" + font_size + "px;";
    div.style.cssText += css_text;
    return div;
  },
  new_div_shape_line_height: function (className, top, left, height, width, font_size, line_height) {
    var div = document.createElement("div");
    div.className = className;
    div.style.cssText = "width:" + width + "px";
    div.style.cssText += "height:" + height + "px";
    div.style.cssText += "top:" + top + "px";
    div.style.cssText += "left:" + left + "px";
    div.style.cssText += "line-height:" + line_height + "px;";
    div.style.cssText += "font-size:" + font_size + "px;";
    return div;
  },

  new_board_shape: function (className, top, col) {
    return this.new_div_shape(className, row, col, this.blank_size, this.piece_font_size);
  },

  new_board_blank: function (row, col, text) {
    var top = row * (this.blank_size + this.border_width);
    var left = col * (this.blank_size + this.border_width);
    var s = null;
      switch (this.name) 
	  {
        case "OC":
          var cols = String.fromCharCode(this.EN_NUM[col].charCodeAt() - 65248);
          var rows = String.fromCharCode(this.STR_NUM[row].charCodeAt() - 65248);
          s = cols + "_" + rows;
          var obj = this.new_div_shape("blank", top, left, this.blank_size, this.blank_size, this.piece_font_size, s);
          obj.onclick = this.shape_mouse_click;
          break;
      }
	  
    return obj;
  },
  new_board_piece: function (row, col, className, text) {
    var top = row * (this.blank_size + this.border_width);
    var left = col * (this.blank_size + this.border_width);
    var div = null;
    var piece_blank = null;
    switch (this.name) {
      case "OC":
        div = this.new_div_shape("piece", -3, -3, this.blank_size, this.blank_size, this.piece_font_size);
        var piece_str = this.new_div_shape(className, -2, -2, this.blank_size, this.blank_size, this.piece_font_size);
        piece_str.append(text);
        div.append(piece_str);
        var r = (9 - row);
        var cols = String.fromCharCode(this.EN_NUM[col].charCodeAt() - 65248);
        var rows = String.fromCharCode(this.STR_NUM[r].charCodeAt() - 65248);
        piece_blank = document.getElementById(cols + "_" + rows);
        piece_blank.appendChild(div);
        return piece_blank;
    }
    return div;
  },
  new_bound: function (row, col, text) {
    var top = row * (this.blank_size + this.border_width);
    var left = col * (this.blank_size + this.border_width);
    var div = this.new_div_shape("boundary", top, left, this.blank_size, this.blank_size, this.bound_font_size);
    div.append(text);
    return div;
  },
  new_score_num: function (className, text, top, left){
	var height = 20 * 1.5;
    var width = (text.length + 2) * 20;
    var div = this.new_div_shape(className, top, left, height, width, 25);
    div.id = className;
	div.append(text); 
    return div;
  },
  new_score_title1: function (text, top, left){
	var height = 20 * 1.5;
    var width = (text.length + 2) * 20;
    var div = this.new_div_shape("score_title", top, left, height, width, 20);
    div.append(text);
    return div;
  },
  new_score_title2: function (text, top, left){
	var height = 20 * 1.5;
    var width = (text.length + 2) * 20;
    var div = this.new_div_shape("score_title", top, left, height, width, 20, "white_name");
    div.append(text);
    return div;
  },
  new_movelist_title: function (text) {
    var height = this.movelist_font_size * 1.5;
    var width = (text.length + 2) * this.movelist_font_size;
    var left = 1 * (this.blank_size + this.border_width);
    var div = this.new_div_shape("label", 0, left, height, width, this.movelist_font_size);
    div.append(text);
    return div;
  },
  new_observation_movelist_title: function (text) {
    var height = this.movelist_font_size * 1.5;
    var width = (text.length + 2) * this.movelist_font_size;
    var left = 1.5 * (this.blank_size + this.border_width);
    var div = this.new_div_shape("label", 0, left, height, width, this.movelist_font_size);
    div.append(text);
    return div;
  },
  new_movelist: function (row, col) {
    var height = row * (this.blank_size + this.border_width);
    var width = col * (this.blank_size + this.border_width);
    var top = this.movelist_font_size * 2;
    return this.new_div_shape("path", top, 0, height, width, height);
  },
  new_movelist_move: function (row, col, type, text) {
    var height = this.movelist_font_size * 1.8;
    var width = (text.length + 1) * this.movelist_font_size;
    var top = row * (this.movelist_font_size) * 2;
    var left = col * (this.blank_size + this.border_width) + this.movelist_font_size * 2 + 3;
    var div_num = this.new_div_shape_line_height("move_num", top, 0, height, this.movelist_font_size * 2, this.movelist_font_size, this.movelist_font_size * 1.3);
    var div_move = this.new_div_shape_line_height(type, top, left, height, width, this.movelist_font_size, this.movelist_font_size * 1.3);
    var div = document.createElement("div");

    div.setAttribute("top", top);
    div_move.append(text);
    div_num.append((row + 1) + ".");
    div.append(div_num);
    div.append(div_move);
    if (this.mode != 1) {
      div.onclick = function () { board.history_move(parseInt(top / 24))};
    }
    return div;
  },
  new_observation_movelist_move: function (row, col, type, text) {
    //console.log("new_observation_movelist_move: " 
    //  + board.player + " " + row + " " + col + " " + type + " " + text);
    var height = this.movelist_font_size * 1.8;
    var width = (text.length + 1) * this.movelist_font_size;
    var top = row * (this.movelist_font_size) * 2;

    if (type != "time1" && type != "time2") {
      var left = 0 * (this.blank_size + this.border_width) + this.movelist_font_size * 2 + 3;
      var div_num = this.new_div_shape_line_height("move_num", top, 0, height, this.movelist_font_size * 2, this.movelist_font_size, this.movelist_font_size * 1.3);
      var div_move = this.new_div_shape_line_height(type, top, left, height, width, this.movelist_font_size, this.movelist_font_size * 1.3);
    } else {

      var left = 2 * (this.blank_size + this.border_width) + this.movelist_font_size * 2 - 3;
      var div_move = this.new_div_shape_line_height(type, top, left, height, 8 * this.movelist_font_size, this.movelist_font_size, this.movelist_font_size * 1.3);
    }

    var div = document.createElement("div");

    div.setAttribute("top", top);
    div_move.append(text);
    if (type != "time1" && type != "time2") {
      div_num.append((row + 1) + ".");
      div.append(div_num);
    }
    div.onclick = function () { board.history_move(parseInt(top / 24)) };
    div.append(div_move);
    return div;
  },
  new_endgame_movelist_move: function (row, col, type, text, end_score, end_type) {
    let _this = this;
    var left = function (col) { return col * (_this.blank_size + _this.border_width) + _this.movelist_font_size * 2 + 3; }
    var height = this.movelist_font_size * 1.8;
    var width = (text.length + 1) * this.movelist_font_size;
    var top = row * (this.movelist_font_size) * 2;
    var div_num = this.new_div_shape_line_height("move_num", top, 0, height, this.movelist_font_size * 2, this.movelist_font_size, this.movelist_font_size * 1.3);
    var div_move = this.new_div_shape_line_height(type, top, left(col), height, width, this.movelist_font_size, this.movelist_font_size * 1.3);
    var div_scr = this.new_div_shape_line_height("end_score", top, left(col + 2), height, this.movelist_font_size * 2, this.movelist_font_size, this.movelist_font_size * 1.3);
    var div_type = this.new_div_shape_line_height("end_type", top, left(col + 2.6), height, this.movelist_font_size * 2, this.movelist_font_size, this.movelist_font_size * 1.3);
    var div = document.createElement("div");
    div.setAttribute("top", top);
    div_move.append(text);
    div_num.append(row + ".");
    div_scr.append(end_score);
    div_type.append(end_type);
    div.append(div_num);
    div.append(div_move);
    div.append(div_scr);
    div.append(div_type);
    return div;
  },
  new_right_arrow: function (row, col, height, width) {
    var top = row * (this.blank_size + this.border_width);
    var left = col * (this.blank_size + this.border_width);
    return this.new_div_shape("right_arrow", top, left, height, width, height);
  },
  new_left_arrow: function (row, col, height, width) {
    var top = row * (this.blank_size + this.border_width);
    var left = col * (this.blank_size + this.border_width);
    return this.new_div_shape("left_arrow", top, left, height, width, height);
  },
  new_number_move: function (row, col, height, width, text) {
    var top = row * (this.blank_size + this.border_width);
    var left = col * (this.blank_size + this.border_width);
    var div = this.new_div_shape("label", top, left, height, width, this.movelist_font_size);
    div.append(text);
    return div;
  },

  toSide: function (pce) {
    return this.PCE_MAP[pce].clr;
  },

  toStr: function (pce) {
    switch (this.LANG) {
      case this.LANG_EN:
        return this.PCE_MAP[pce].en;
      case this.LANG_CH:
        return this.PCE_MAP[pce].ch;
    }
  },

  toClassName: function (side) {
    return this.PCE_TYPE[side].className;
  },
  set_board: function (board) { }
}

BaseBoard.prototype.init = function () {
  //console.log("init: " + this.name, this.mode);
  var link = document.createElement('link');
  link.href = "chess.css";
  link.rel = "stylesheet";
  var tagN = document.getElementsByTagName("link");
  if (tagN.length == 0) {
    document.getElementsByTagName('head')[0].append(link);
  }
  this.screen_init();
  this.board_init();
  this.chess_board_init();

  this.piece_init();

  this.piece_score_init(2,2);

  this.div_movelist.append(this.new_observation_movelist_title("走步"));
  this.div_path = this.new_movelist(this.piece_row, 5);

  this.div_movelist.append(this.div_path);
  //console.log(typeof document.body, this.mode);

  document.body.onkeydown = this.keyEvent;
  document.body.onkeyup = this.metaKeyUp;

  //if( !this.mode ){add_input();}
  /*var bt = document.createElement("input");
  bt.id = "hisbtn";
  bt.type = "button";
  bt.value = "歷史走步";
  bt.onclick = function(){window.open('history.html', 'History', config='height=500,width=800');}
  var bget = this.div_endgame_movelist;
  bget.append(bt);*/
  //console.log("init end");
};

function his_init(index) {
  //console.log("his index", index, board.mode);
  if (board.mode == 1 && board.isstopboard == false) { return; }
  if (document.getElementById("set_mode") != null &&
    document.getElementById("set_mode").checked == false) {
    return;
  }
  let turn = board.turn;
  let stop_flag = board.isstopboard;
  let openf = board.his_openf;
  let clr = board.clr;
  let buf = board.actlist;
  let tbuf = board.acttime;
  let data = board.board_data;
  let mode = board.mode;
  let info_r = board.info_row;
  let infoerr_r = board.info_error_row;
  let ply = board.player;
  var players = board.player_name;
  var sock = board.socket;
  let div_endgame = document.getElementById("div_endgame_movelist_OC");
  let mui = document.getElementById("mui_sw");
  let mui1 = document.getElementById("mui_sw1");
  let f = true; let f1 = true;

  if (mui != null) {
    f = mui.checked;
    f1 = mui1.checked;
  }

  board = null;
  document.body.innerHTML = "";
  document.body.setAttribute("overflow-x", "hidden");
  document.body.setAttribute("overflow-y", "hidden");
  board = new OCBoard(mode);
  board.turn = turn;
  board.isstopboard = stop_flag;
  board.socket = sock;
  board.clr = clr;
  board.step = index - 1;
  board.his_step = index;
  board.actlist = buf;
  board.player = ply;
  board.his_openf = openf;
  board.player_name = players;

  //console.log("color", board.clr);
  // history
  if (board.mode == 0 || board.mode == 4) {
    let sw_lab = document.createElement("span");
    sw_lab.textContent = "使用者標準輸出自動換行:";
    sw_lab.style = "float: left;";

    let sw_lab1 = document.createElement("span");
    sw_lab1.textContent = "使用者標準錯誤輸出自動換行:";
    sw_lab1.style = "float: left;";

    let sw = document.createElement("input");
    sw.id = "mui_sw";
    sw.className = "mui-switch mui-switch-animbg";
    sw.type = "checkbox";
    sw.checked = f;

    let sw1 = document.createElement("input");
    sw1.id = "mui_sw1";
    sw1.className = "mui-switch mui-switch-animbg";
    sw1.type = "checkbox";
    sw1.checked = f1;

    sw_lab.appendChild(sw);
    sw_lab1.appendChild(sw1);
    board.div_endgame_movelist.insertBefore(sw_lab1, board.div_endgame_movelist.childNodes[0]);
    board.div_endgame_movelist.insertBefore(sw_lab, board.div_endgame_movelist.childNodes[0]);
    board.acttime = tbuf;
    board.board_data = data;
    board.info_row = info_r;
    board.info_error_row = infoerr_r;
    while (div_endgame.firstChild) {
      let clone = div_endgame.firstChild.cloneNode(false);
      board.div_endgame_movelist.appendChild(clone);
      //console.log(clone);
      div_endgame.removeChild(div_endgame.firstChild);
    }
    board.create_div_open_save();

    let sp = document.createElement("span");
    sp.append(openf);
    sp.style.float = "center";
    board.div_endgame_movelist.insertBefore(sp, board.div_endgame_movelist.childNodes[0]);

    window.document.title = openf;
  }
  // abort
  if (board.mode == 2) {
    let abort_clone = div_endgame.firstChild.cloneNode(true);
    board.div_endgame_movelist.appendChild(abort_clone);
  }
  //observation
  if (board.mode == 3 || board.isstopboard) {
    while (div_endgame.firstChild) {
      let _clone = div_endgame.firstChild.cloneNode(true);
      div_endgame.removeChild(div_endgame.firstChild);
      board.div_endgame_movelist.appendChild(_clone);
    }
    document.getElementById("set_mode").onchange = CBclick;
  }
  if (document.getElementById("message_button") != null) {
    document.getElementById("message_button").addEventListener("click", get_Chat_Content);
  }
  if (document.getElementById("message_input") != null) {
    document.getElementById("message_input").addEventListener("keyup", function (event) {
      // Number 13 is the "Enter" key on the keyboard
      if (event.keyCode === 13) {
        // Cancel the default action, if needed
        event.preventDefault();
        // Trigger the button element with a click
        document.getElementById("message_button").click();
      }
    });
  }

  buf = null;
  tbuf = null;
  data = null;
  clr = null;
  info = null;
  var move = null;
  var move_pos = null;
  var latt = null;
  var ratt = null;
  for (var i = 0; i < board.actlist.length; i++) {
    var strl = board.actlist[i];
    let text = strl.split(" ");
    var num = (i == index) ? 2 : 0;
    strl = strl.split(' ')[1];
    //console.log("now step:", i);
    board.div_path.append(
      move = board.new_observation_movelist_move(i, (i % 2 == 0) ? 0 : 2,
        "move" + ((board.clr + i) % 2 + 1 + num), text[1])
    );
    board.div_path.append(
      board.new_observation_movelist_move(i, (i % 2 == 0) ? 0 : 2,
        "time" + ((board.clr + i) % 2 + 1),
        ((parseInt(text[2]) / 1000).toFixed(1) + " / " + (parseInt(text[3]) / 1000).toFixed(1)))
    );

    if (i == (index - 1)) { latt = move.getAttribute("top"); }
    if (i == (index + 1)) { ratt = move.getAttribute("top"); }
    if (i == (index - 7)) { move_pos = move.getAttribute("top"); }
  }

  board.div_movelist.append(board.new_left_arrow(board.piece_row + 1, 0, 18, 30));
  board.div_movelist.append(board.new_right_arrow(board.piece_row + 1, 3, 18, 30));
  var la = board.div_movelist.getElementsByClassName("left_arrow")[0];
  var ra = board.div_movelist.getElementsByClassName("right_arrow")[0];
	
  board.div_movelist.append(board.new_number_move(board.piece_row + 1, 1,
    board.movelist_font_size * 1.8, board.movelist_font_size * 5,
    "" + (index + 1) + "/" + (board.actlist.length))
  );



  if (latt != null) {
    la.setAttribute("top", latt);
    la.onclick = function () {
      board.history_move(parseInt(latt / 24))
    };
  } else {
    //console.log(la, board.actlist.length - 1);
    la.setAttribute("top", board.actlist.length - 1);
    la.onclick = function () {
      board.history_move(parseInt(board.actlist.length - 1));
    };
  }
  if (ratt != null) {
    ra.setAttribute("top", ratt);
    ra.onclick = function () { board.history_move(parseInt(ratt / 24)) };
  } else {
    //console.log(ra, board.actlist.length - 1);
    ra.setAttribute("top", 0);
    ra.onclick = function () {
      board.history_move(parseInt(0));
    };
  }

  board.div_path.scrollTop = move_pos;
  la = null; ra = null;

  //history
  if (board.mode == 0 || board.mode == 4) {
    setPos(index + 1);
    var vs = new Array();
    if (board.board_data == "") {
      //console.log(board.board_data);
      vs[0] = "先手", vs[2] = "後手";
    } else {
      vs = board.board_data.split("* ")[2].split(" ");
    }
    let name_lab1 = document.createElement("span");
    name_lab1.textContent = vs[0];
    name_lab1.style.float = "left";
    name_lab1.style.backgroundColor = (board.clr == 2) ? "black" : "white";
    name_lab1.style.color = "black";
    name_lab1.style.fontSize = "18px";

    let lab_vs = document.createElement("span");
    lab_vs.textContent = "vs";
    lab_vs.style.paddingLeft = "10px";
    lab_vs.style.paddingRight = "10px";
    lab_vs.style.float = "left";
    lab_vs.style.color = "black";
    lab_vs.style.fontSize = "18px";


    let name_lab2 = document.createElement("span");
    name_lab2.textContent = vs[2];
    name_lab2.style.float = "left";
    name_lab2.style.backgroundColor = (board.clr == 1) ? "white" : "black";
    name_lab2.style.color = "white";
    name_lab2.style.fontSize = "18px";

    board.div_endgame_movelist.insertBefore(name_lab2, board.div_endgame_movelist.childNodes[0]);
    board.div_endgame_movelist.insertBefore(lab_vs, board.div_endgame_movelist.childNodes[0]);
    board.div_endgame_movelist.insertBefore(name_lab1, board.div_endgame_movelist.childNodes[0]);
  }
  if (board.mode == 2) {
    //console.log(div_endgame);
    let box = document.getElementById("box_dialog");
    let text_step = 1;
    var yn = "no";
    //console.log(box);
    if (box != null) {
      //console.log("box exist");
      let text = document.getElementById("abort_step");
      if (text != null) {
        text.value = index + 1;
        text_step = text.value;
      }
      let r = document.getElementById("radio_1");
      if (r != null) {
        if (r.checked == true) {
          yn = "yes";
        }
      }
      //console.log("悔棋", "雙方是否交換顏色?", text_step, yn, r.checked, r);
      box.remove();
      DEBUG_SET("悔棋", "雙方是否交換顏色?", text_step, yn);
    } else {
      //console.log("box null");
      text_step = document.getElementsByClassName("path")[0].childNodes.length;
      DEBUG_SET("悔棋", "雙方是否交換顏色?", text_step, yn);
    }
  }
  window = board;
}


var posarr = [0, 0];
// set textarea position
function setPos(ind) {
  //console.log("setPos", ind);

  //scrollHeight all
  //clientHeight see
  let sw = document.getElementById("mui_sw");
  let sw1 = document.getElementById("mui_sw1");
  let textArea = document.getElementById("user_info");
  let textArea1 = document.getElementById("info_error");
  if (sw.checked == true) {
    let lineHeight = textArea.scrollHeight / board.info_row[board.info_row.length - 2];
    textArea.scrollTop = lineHeight * board.info_row[ind];
    posarr[0] = textArea.scrollTop;
  } else {
    textArea.scrollTop = posarr[0];
  }
  if (sw1.checked == true) {
    let lineHeight1 = textArea1.scrollHeight / board.info_error_row[board.info_error_row.length - 2];
    textArea1.scrollTop = lineHeight1 * board.info_error_row[ind];
    posarr[1] = textArea1.scrollTop;
  } else {
    textArea1.scrollTop = posarr[1];
  }
}
function focus_textbox() {
  //console.log("textbox");
  board.his_create_name = true;
}
function blur_textbox() {
  board.his_create_name = false;
}

function _Move(sta_p, playerNum) {

 // console.log(board.PCE_MAP[board.actlist[0].split(" ")[1][2]]);
  //console.log(board.actlist[0].split(" "));
  var sta = document.getElementById(sta_p);
  //console.log(sta);
 // console.log(sta + ", "+ playerNum);

  //var pie = sta.firstElementChild;
  if(!!sta){
	  while (sta.firstChild) {
    sta.removeChild(sta.firstChild);
  }
  var div = document.createElement("div");
  if(playerNum %2 == 0)  
  {
	  //board.clr = 1;
	  div.className = "piece_black";
  }
  else
  {
	  //board.clr = 2;
	  div.className = "piece_white";
  }
	
  div.style.cssText = "width: 50px";
  div.style.cssText += "height: 50px";
  div.style.cssText += "top: -2px";
  div.style.cssText += "left: -2px";
//console.log(div);
  sta.append(div);
	
  div = null;
  div = document.createElement("div");
  div.className = "orange_rect";
  div.style.cssText = "width: 50px";
  div.style.cssText += "height: 50px";
  div.style.cssText += "top: -2px";
  div.style.cssText += "left: -2px";
  sta.append(div);
  board.tmp = null;
  board.after_blank = null;
  board.remove_div("rect");
  }
}
// create save div
BaseBoard.prototype.create_div_open_save = function () {
  let bget = board.div_endgame_movelist;
  bget.style.width = "50%";
  let div = document.createElement("div");
  var style = "width:100%;";
  style += "position: relative;";
  style += "float: left;";
  div.setAttribute("style", style);
  let fopen = document.createElement("input");
  fopen.id = "myfile";
  fopen.type = "file";
  fopen.accept = ".txt";
  fopen.onchange = Open;
  fopen.style = "float: left;";

  let fname = document.createElement("input");
  fname.id = "filename";
  fname.type = "text";
  fname.style = "float: left;";
  fname.onclick = focus_textbox;
  fname.onblur = blur_textbox;

  let btn = document.createElement("input");
  btn.id = "submit";
  btn.value = "save";
  btn.type = "button";
  btn.style.top = "400px";
  btn.onclick = Save;
  btn.style = "float: left;";
  if (board.mode == 0) {
    div.appendChild(fopen);
  }
  div.appendChild(fname);
  div.appendChild(btn);
  bget.appendChild(div);

  bget.insertBefore(div, bget.childNodes[0]);
}

// create information areatext
function add_input() {
  board.create_div_open_save();
  //console.log("add_input");
  let bget = board.div_endgame_movelist;
  let uinfo = document.createElement("textarea");
  uinfo.id = "user_info";

  style = "";
  style += "float: left;";
  style += "font-family: Courier New;";
  style += "position: relative;";
  uinfo.setAttribute("style", style);

  uinfo.style.width = "100%";
  uinfo.style.maxWidth = "100%";
  uinfo.style.resize = "none";
  uinfo.style.overflowX = "scroll";
  uinfo.style.overflowY = "scroll";
  uinfo.style.whiteSpace = "nowrap";
  uinfo.rows = "20";

  let uinfoerr = document.createElement("textarea");
  uinfoerr.id = "info_error";

  style = "";
  style += "float: left;";
  style += "font-family: Courier New;";
  style += "position: relative;";
  uinfoerr.setAttribute("style", style);


  uinfoerr.style.width = "100%";
  uinfoerr.style.maxWidth = "100%";
  uinfoerr.style.resize = "none";
  uinfoerr.style.overflowX = "scroll";
  uinfoerr.style.overflowY = "scroll";
  uinfoerr.style.whiteSpace = "nowrap";
  uinfoerr.rows = "20";

  bget.appendChild(uinfo);
  bget.appendChild(uinfoerr);
}
function remove_px(text) {
  var str = text.split('px');
  return parseInt(str[0]);
}
BaseBoard.prototype.keyEvent = function (event) {

  var key = event.keyCode || event.which;
  var keychar = String.fromCharCode(key);
  //console.log("keyEvent", key);
  //console.log(board.his_create_name, "key");
  if (board.mode == 1 && board.isstopboard == false) {return; }

  if (key == this.exampleKey) {
    this.metaChar = true;
  }
  if (key != this.exampleKey) {
    if (this.metaChar) {
      this.metaChar = false;
    } else {
      //console.log(key);
      switch (key) {
        case 65:
        case 37:
        case 87:
        case 38:
          if (parseInt(board.his_step - 1) >= 0) {
            board.history_move(parseInt(board.his_step - 1));
          } else {
            //max step
            board.history_move(parseInt(board.actlist.length - 1));
          }
          break;
        case 68:
        case 39:
        case 83:
        case 40:
          if (parseInt(board.his_step + 1) < board.actlist.length) {
            board.history_move(parseInt(board.his_step + 1));
          } else {
            //first step
            board.history_move(parseInt(0));
          }
          break;
      }
    }
  }
}
BaseBoard.prototype.metaKeyUp = function (event) {
  //console.log("metaKeyUp");
  var key = event.keyCode || event.which;
  if (key == this.exampleKey) {
    this.metaChar = false;
  }
}

BaseBoard.prototype.history_move = function (item) {
  //var item = parseInt(this.getAttribute("top")) / 24 ;
  if (document.getElementById("set_mode") != null &&
    document.getElementById("set_mode").checked == false) {
    return;
  }
  console.log("history_move", item); 
  his_init(item);
  var moveid = -1;
  model1 = new Model();
  while (moveid < item) {
    board.remove_div("blue_rect");
    board.remove_div("black_rect");
    board.remove_div("white_rect");
    board.remove_div("orange_rect");

    moveid++;
    let list = board.actlist[moveid];
    let str = list.split(' ')[1];

    console.log(str);

    
    //remove_rect();
	var stanum = ((str[0].charCodeAt() - 'a'.charCodeAt()) * 8)+ (str[1].charCodeAt() - '0'.charCodeAt());
	var staid = "" + str[0] + "_" + str[1];
	_Move(staid, moveid%2);
	if(str[0] != '-')
		MakeMove_H(str[0].charCodeAt() - 'a'.charCodeAt(),str[1].charCodeAt() - '1'.charCodeAt());
  }
}
BaseBoard.prototype.board_init = function () {
  //console.log("BB board_init");
  this.div_board.append(this.new_div_shape("boardBk",13, 10, this.blank_size*10+5, this.blank_size*10+5));
  for (var row = 1; row <= this.blank_row; row++) {
    for (var col = 1; col <= this.blank_col; col++) {
      this.div_board.append(this.new_board_blank(row, col));
    }
  }
}

BaseBoard.prototype.path_init = function (move, start, end, ply, this_ply) {
}


BaseBoard.prototype.movelist_init = function (moves, ply, url_first, url_last) {
  this.div_movelist.append(this.new_movelist_title("走步", 1));
  var new_url = function (_ply) { return url_first + (_ply) + url_last; }
  this.div_path = this.new_movelist(this.piece_row, 4);
  var row = 0;
  var move = null;
  var move_pos = 0;
  var max_ply = moves.length;

  //console.log("JJJJ"+this.PCE_MAP[moves[0][3]].clr);
  var clr_1, clr_2;
  if (this.name == "OC") {
    clr_1 = (this.PCE_MAP[moves[0][3]].clr == "WHITE" ? 3 : 4);
    clr_2 = (this.PCE_MAP[moves[0][3]].clr == "WHITE" ? 1 : 2);
  }
 
  var option = "";
  for (var i = 0; i < max_ply; i++) {
    let url = new_url(i + 1);
    this.div_path.append(move = this.new_movelist_move(i, i % 2, "move" + (i == (ply - 1) ? clr_1 + i % 2 : clr_2 + i % 2), moves[i]));
    if ((i == (ply - 1) ? clr_1 + i % 2 : clr_2 + i % 2) > 2) { this.player = i; option = moves[i]; }
    move.onclick = function () { location.href = url; };
    if (i == (ply - 7)) move_pos = move.getAttribute("top");//style.top.substring(0, move.style.top.length - 2);
  }
  if (this.name == "OC") {
    var en_pos = String.fromCharCode(option.charCodeAt(0) + 65248);
    var nu_pos = String.fromCharCode(option.charCodeAt(1) + 65248);

	var element = document.getElementById(en_pos + "_" + nu_pos);
	var div = document.createElement("div");
	div.className = "move_mark";
	div.style.backgroundColor = "blue";
	div.style.left = "34px";
	div.style.top = "34px";
	element.append(div);  
  }
  this.div_movelist.append(this.div_path);
  this.div_movelist.append(this.new_left_arrow(this.piece_row + 1, 0, 18, 30));
  this.div_movelist.append(this.new_right_arrow(this.piece_row + 1, 3, 18, 30));
  var str = ply.toString() + "/" + max_ply.toString();
  this.div_movelist.append(this.new_number_move(this.piece_row + 1, 1, this.movelist_font_size * 1.8, this.movelist_font_size * 5, str));
  this.div_movelist.getElementsByClassName("left_arrow")[0].onclick = function () { location.href = new_url(Math.max(0, ply - 1)); }
  this.div_movelist.getElementsByClassName("right_arrow")[0].onclick = function () { location.href = new_url(Math.min(max_ply, ply + 1)); }
  this.div_path.scrollTop = move_pos;
}

BaseBoard.prototype.piece_score_init = function (black_score, white_score)
{
	this.div_piece_score.append(this.new_score_title1("黑子", 130, 0));
	this.div_piece_score.append(this.new_score_num("black_score",black_score, 177, 0, 100, 100));
	this.div_piece_score.append(this.new_score_title2("白子", 230,0));
	this.div_piece_score.append(this.new_score_num("white_score",white_score, 277, 0, 100, 100));
	
}

BaseBoard.prototype.endgame_movelist_init = function (best_score, moves, best_index, url) {
  //console.log(moves);
  this.div_endgame_movelist.append(this.new_movelist_title("殘局", 1));
  var new_url = function (HIS) { return url + HIS; }
  //this.div_path = this.new_movelist(this.piece_row, 4);
  var row = 0;
  var move = null;
  var move_pos = 0;
  var max_ply = moves.length;

  var div_path = this.new_movelist(this.piece_row, 4);

  for (var i = 0; i < moves.length; i++) {
    let url = new_url(moves[i]);
    let s = moves[i].split(" ");
    let class_name = "end_move" + (i == best_index ? "_best" : "");
    div_path.append(move = this.new_endgame_movelist_move(i, 0, class_name, s[0], s[1], s[2]));
    move.onclick = function () { location.href = new_url(s[3]); };
    //if (i == (ply - 7)) move_pos = move.getAttribute("top");//style.top.substring(0, move.style.top.length - 2);
  }
  this.div_endgame_movelist.append(div_path);
  this.div_endgame_movelist.append(this.new_left_arrow(this.piece_row + 1, 0, 18, 30));
  this.div_endgame_movelist.append(this.new_right_arrow(this.piece_row + 1, 3, 18, 30));
  var str = best_score;//ply.toString() + "/" + max_ply.toString();
  this.div_endgame_movelist.append(this.new_number_move(this.piece_row + 1, 1, this.movelist_font_size * 1.8, this.movelist_font_size * 5, str));
  div_path.scrollTop = move_pos; //this.div_path.scrollHeight;
  this.div_endgame_movelist.getElementsByClassName("left_arrow")[0].onclick = function () { location.href = new_url(moves[best_index].split(" ")[3]); }
  this.div_endgame_movelist.getElementsByClassName("right_arrow")[0].onclick = function () { location.href = new_url(moves[best_index].split(" ")[3]); }
  //console.log(move_pos);
}



BaseBoard.prototype.set_status = function (moves, ply, prev_url, next_url) {
}


function OCBoard(mode) {
  BaseBoard.call(this, 8, 8, "OC", mode);
  this.init();
}
OCBoard.prototype = new BaseBoard();

OCBoard.prototype.chess_board_init = function () {
  //console.log("chess_board_init");
  for (j = 1, i = 8; j <= 8; j++, i--)
    this.div_board.append(this.new_bound(j, 0, this.STR_NUM[j]));
  for (i = 1; i <= 8; i++)
    this.div_board.append(this.new_bound(0, i, this.EN_NUM[i]));
  //console.log("chess_board_init end");
}

OCBoard.prototype.piece_init = function () {
  //console.log("piece_init");
	
  for (var row = 1; row <= this.piece_row; row++) {
    for (var col = 1; col <= this.piece_col; col++) {
	  if((row == 4 && col == 4)||(row == 5 && col == 5))
		this.div_piece.append(this.new_board_piece(row, col, this.toClassName("BLACK"), ""));
	  else if((row == 4 && col == 5)||(row == 5 && col == 4))
		this.div_piece.append(this.new_board_piece(row, col, this.toClassName("WHITE"), ""));
    }
  }
  //console.log("piece_init end");

}

OCBoard.prototype.screen_init = function () {
  //console.log("screen_init");
  var style = "";
  style = "";
  style += "top :" + (0 * (this.blank_size + this.border_width) + this.boundary_top) + "px;"
  style += "left:" + (3 * (this.blank_size + this.border_width) + this.boundary_left) + "px;"
  this.div_board.setAttribute("style", style);

  style = "";
  style += "top :" + (0 * (this.blank_size + this.border_width) + this.piece_bound_top + this.boundary_top) + "px;"
  style += "left:" + (3 * (this.blank_size + this.border_width) + this.piece_bound_left + this.boundary_left) + "px;"
  this.div_piece.setAttribute("style", style);

  style = "";
  style += "top :" + ((0 * (this.die_blank_size + this.border_width)) + (0.8 * (this.blank_size + this.border_width)) + this.boundary_top) + "px;"
  style += "left:" + (0.5 * (this.blank_size + this.border_width) + this.boundary_left) + "px;"
  this.div_piece_score.setAttribute("style", style);

  style = "";
  style += "top :" + (0.5 * (this.blank_size + this.border_width) + this.boundary_top) + "px;"
  style += "left:" + (14.5 * (this.blank_size + this.border_width) + this.boundary_left) + "px;"
  this.div_movelist.setAttribute("style", style);

  style = "";
  style += "top :" + (0.5 * (this.blank_size + this.border_width) + this.boundary_top) + "px;"
  style += "left:" + (20.5 * (this.blank_size + this.border_width) + this.boundary_left) + "px;"
  this.div_endgame_movelist.setAttribute("style", style);
  //console.log("screen_init end");
}

/* mouse event */
var add_value = [-1, 1];
var push_in = -1;
var push_s = "";
var S_NUM = ["０", "１", "２", "３", "４", "５", "６", "７", "８"];
var E_NUM = ["", "ａ", "ｂ", "ｃ", "ｄ"];
var S_NUM_url = ["0", "1", "2", "3", "4", "5", "6", "7", "8"];
var E_NUM_url = ["", "a", "b", "c", "d", "e", "f", "g", "h"];

var PCE_POS = new Map();

var pie_tmp = null;
var after_blank = null;
/*
var Score_map = set_score();
function set_score() {
  var s_map = new Map(score);
  //.log(s_map);
  return s_map;
}*/
function create_div(w, h, t, l) {
  var div = document.createElement("div");
  div.className = "rect";
  //div.className = "move_mark";
  div.style.cssText = "width: " + w + "px";
  div.style.cssText += "height: " + h + "px";
  div.style.cssText += "top: " + t + "px";
  div.style.cssText += "left: " + l + "px";
  //console.log("create_div end", w, h, t, l);
  return div;
}

var Model = function()
{
	this.black =2;
	this.white =2;
	this.result;
	this.lastPass = false;
	this.gc = false;
	this.moveCounter =0;
	this.playerId = 1+this.moveCounter%2;
	this.cellArray =[
		[0,0,0,0,0,0,0,0],
        [0,0,0,0,0,0,0,0],
        [0,0,0,0,0,0,0,0],
        [0,0,0,2,1,0,0,0],
        [0,0,0,1,2,0,0,0],
        [0,0,0,0,0,0,0,0],
        [0,0,0,0,0,0,0,0],
        [0,0,0,0,0,0,0,0]
	];
}

Model.prototype.gameInfo = function(){
	if(gc)
	return result;
	return "Player "+ playerId +" should play"
}

const colors = ["green","black","white","grey"];

Model.prototype.setColor = function(row,col) {
	return colors[this.cellArray[row][col]];
}

Model.prototype.showPossibleMoves = function(){
	for(let row=0;row<8;row++){
		for(let col=0;col<8;col++){
			for(let dir=0;dir<8;dir++){
				this.findMoves(row,col,dir);
			}
		}
	}
	var b=0,w=0,p=0,c;
	for(let row=0;row<8;row++){
		for(let col=0;col<8;col++){
			c = this.cellArray[row][col];
			if(c==1){
				b++;
			}else if(c==2){
				w++;
			}else if(c==3)
				p++;
		}
	}
	this.white = w;
	this.black = b;
    //console.log(p);
	if(p==0&&!this.lastPass){
		//console.log("HI");
		//this.lastPass = true;
		if(this.moveCounter <60){
			this.moveCounter++;
			this.playerId = 1+ this.moveCounter%2;
			this.showPossibleMoves();
		}	
		return;
	}
	/*
	else if(p==0&&this.lastPass){
		console.log("gameover");
		this.gc = true;
		if(w>b){
			this.result = "Player 2 Won";
		}else if(b>w){
			this.result = "Player 1 Won";
		}
		else{
			this.result = "Its a Tie-Game";
		}
	}*/
}

var human = false;
//var turn = 0;
Model.prototype.findMoves = function(r,c,dir){
	if(this.cellArray[r][c]!= this.playerId)
	return;
	var row = r;
	var col = c;
	var changeRow = [-1,0,1,1,1,0,-1,-1] ;
	var changeCol = [-1,-1,-1,0,1,1,1,0] ;  
	var cRow = changeRow[dir];
	var cCol = changeCol[dir];
	var other = 1+(1+this.moveCounter)%2;
	row += cRow ;
	col += cCol ; 
	if(row>=0&&row<8&&col>=0&&col<8){
	if(this.cellArray[row][col] ==0){
		return ;
	}}else{
		return ;
	}
	while(row>=0&&row<8&&col>=0&&col<8){
	   if(other==parseInt(this.cellArray[row][col])){
			row += cRow ;
			col += cCol ; 
			}
		else{
			break ;
		}
	}
	if(row<0||row>7||col<0||col>7){
		return ;
	}
	if( this.cellArray[row][col] ==0 ){
	//console.log(r+" "+c+" "+dir);
	this.cellArray[row][col] = 3;
	}
}
Model.prototype.hidePossibleMoves = function(){
	for(let row=0;row<8;row++){
		for(let col=0;col<8;col++){
			if(this.cellArray[row][col]==3)
				this.cellArray[row][col] = 0;
		}
	}
}

Model.prototype.toFlip = function (r,c,dir,p){
	if(this.cellArray[r][c]!=p)
	return 0;
	var other = 1+(p)%2;
	var count = 0;
	var row = r;
	var col = c;
	var changeRow = [-1,0,1,1,1,0,-1,-1] ;
	var changeCol = [-1,-1,-1,0,1,1,1,0] ;  
	var cRow = changeRow[dir];
	var cCol = changeCol[dir];
	row += cRow ;
	col += cCol ; 
	if(row>=0&&row<8&&col>=0&&col<8){
	if(this.cellArray[row][col] ==0){
		return 0 ;
	}}else{
		return 0;
	}
	while(row>=0&&row<8&&col>=0&&col<8){
	   if(other==parseInt(this.cellArray[row][col])){
			row += cRow ;
			col += cCol ; 
			count++;
			}
		else{
			break ;
		}
	}
	if(row<0||row>7||col<0||col>7){
		return 0;
	}
	if( this.cellArray[row][col] ==p){
	//console.log(r+" "+c+" "+dir);
	return count ;
	}
	return 0;
}

Model.prototype.flipCoins = function (r,c,dir,count){
	var changeRow = [-1,0,1,1,1,0,-1,-1] ;
	var changeCol = [-1,-1,-1,0,1,1,1,0] ; 
	cRow = changeRow[dir]   ;
	cCol = changeCol[dir]  ;
	for(let i=0;i<count;i++){
		r += cRow ;
		c += cCol;
		this.cellArray[r][c] = 1+( this.cellArray[r][c] )%2;
		flipPiece(r,c);
   }       
}

Model.prototype.checkMid = function  (p){
	var count,result = false;;
	for(let row=0;row<8;row++){
		for(let col=0;col<8;col++){
			for(let dir=0;dir<8;dir++){
				count = this.toFlip(row,col,dir,p);
				if(count !=0) {
				 //console.log(row+" "+col+" "+dir)  ;
				  this.flipCoins(row,col,dir,count);
					result = true;
				}
			}
		}
	} 
	return result ;   
}

Model.prototype.implementMove = function(row,col){
	for(let i=0;i<8;i++){
		var count = this.toFlip(row,col,i,(1+(1+this.moveCounter)%2));
		if(count !=0){
			this.flipCoins(row,col,i,count);
		}
   }  
   var i=this.playerId;  
   while(0){
  // console.log("loop");
   if(this.checkMid(i)==false &&
   this.checkMid(1+(++i)%2) ==false)
   break ;
   } 
}

Model.prototype.doMove = function(col,row){
	if(this.cellArray[row][col] != 3)
		return ;
	this.cellArray[row][col] = this.playerId;
	this.moveCounter++;
	this.playerId = 1 + this.moveCounter%2;
	this.hidePossibleMoves();
	this.implementMove(row,col);
	this.showPossibleMoves();
}
var reset = true;
var model= new Model();
function MakeMove(row, col)
{
	//board.clr = (board.PCE_MAP[board.actlist[0].split(" ")[1][3]].clr == "BLACK") ? 1 : 2;
	if(reset)
	{
		delete(model);
		model = new Model();
		reset = false;
	}
	model.showPossibleMoves();
	model.doMove(row, col);
	 var b = document.getElementById("black_score");
	 var w = document.getElementById("white_score");
	 b.innerHTML = model.black;
	 w.innerHTML = model.white;
}
var model1= new Model();
function MakeMove_H(row, col)
{	
	if(reset)
	{
		delete(model1);
		model1 = new Model();
		reset = false;
	}
	model1.showPossibleMoves();
	model1.doMove(row, col);
	 var b = document.getElementById("black_score");
	 var w = document.getElementById("white_score");
	 b.innerHTML = model1.black;
	 w.innerHTML = model1.white;
}
var num_E = ["a","b","c","d","e","f","g","h"];
function flipPiece(row, col)
{
	var sta_p = num_E[col] + "_" + (row+1);
	var sta = document.getElementById(sta_p);
	//console.log(sta);
	var pie = sta.firstElementChild;
	while (sta.firstChild) {
		sta.removeChild(sta.firstChild);
	}
	var div = document.createElement("div");
	if(model.cellArray[row][col] == 1)
		div.className = "piece_black";
	else
		div.className = "piece_white";
	div.style.cssText = "width: 50px";
	div.style.cssText += "height: 50px";
	div.style.cssText += "top: -2px";
	div.style.cssText += "left: -2px";
	sta.append(div);		
}

function legalPositionShow()
{
	model.showPossibleMoves();
	model1.showPossibleMoves();
	for(i=0; i<8;i++)
		for(j=0; j<8; j++)
		{
			if(model.cellArray[i][j] == 3)
			{
				var sta_p = num_E[j] + "_" + (i+1);
				var sta = document.getElementById(sta_p);
				//console.log(sta);
				var pie = sta.firstElementChild;
				if(human && board.turn == 1)
				{
					div = document.createElement("div");
					div.className = "blue_rect";
					div.style.cssText = "width: 50px";
					div.style.cssText += "height: 50px";
					div.style.cssText += "top: -2px";
					div.style.cssText += "left: -2px";
					sta.append(div);
				}
			}
		}
	
}
function RestModel()
{
	model = new Model();
	model1 = new Model(); 
}
BaseBoard.prototype.remove_div = function (name) {
  var element = document.getElementsByClassName(name);
  while (element.length > 0) { element[0].remove(); }
}

OCBoard.prototype.piece_click = function () {
  //console.log("OC click ", board.player, board.tmp);
  //console.log("piece_click", this);
  remove_rect();
  if (board.turn == 0 || board.mode != 1) { return; }

  if (this.firstElementChild.className.split('_')[1] == "white" && board.player % 2 == 1) {
    return;
  }
  else if (this.firstElementChild.className.split('_')[1] == "black" && board.player % 2 == 0) {
    return;
  }

  if (board.tmp != null) {
    board.tmp.style.backgroundColor = "#a5a4a4";
  } else {
    board.tmp = this;
    board.tmp.style.backgroundColor = "#a5a4a4";
  }

  var pathnode = (board.div_path.childNodes.length > 0) ? true : false;
  //console.log("board.clr", board.clr);
  var canm = false;
  if (pathnode) {
    //console.log(board.div_path.lastChild.lastChild.className);
    let path_node = board.div_path.childNodes[board.div_path.childNodes.length - 1];
    //white :move1, move3
    //console.log("board.div_path：",board.div_path.childNodes[board.div_path.childNodes.length-2] );
    if (path_node.lastChild.className == "time1") { canm = true; }
    if (path_node.lastChild.className == "move1") { canm = true; }
    if (path_node.lastChild.className == "move3") { canm = true; }
    //black :move2, move4
    if (path_node.lastChild.className == "time2") { canm = false; }
    if (path_node.lastChild.className == "move2") { canm = false; }
    if (path_node.lastChild.className == "move4") { canm = false; }

  }

  //console.log(canm, "---------",this.firstElementChild.className, board.clr, board.tmp);
  switch (this.firstElementChild.className) {
    case "piece_white":
      var childNode = this.parentNode.childNodes;
      //var canm = (board.clr % 2) == (board.player % 2)? true: false;
      //var canm = (board.clr == 0) ? true: false;
      if (!canm && childNode.length < 2) {
        board.remove_div("rect");
        this.style.backgroundColor = "#60d386";
        board.tmp = this;
      }
      if (canm && childNode.length < 2) { board.remove_div("rect"); }
      break;
    case "piece_black":
      var childNode = this.parentNode.childNodes;
      //var canm = (board.clr % 2 == 1) == (board.player % 2 == 0)? true: false;
      //var canm = (board.clr == 1) ? true: false;
      if (canm && childNode.length < 2) {
        board.remove_div("rect");
        this.style.backgroundColor = "#60d386";
        board.tmp = this;
      }
      if (!canm && childNode.length < 2) { board.remove_div("rect"); }
      break;
  }
}
OCBoard.prototype.shape_mouse_click = function () {
  //console.log("shape mouse click ", board.turn, board.clr, this, board.tmp);
  if (board.mode != 1 || board.turn == 0 ) { return; }

  if (board.turn == 1) {
    if (this.childNodes.length > 0) {
      var b_children = this.childNodes;
      var is_rect = b_children[b_children.length - 1];
	  //console.log(is_rect);
      if (is_rect.className == "blue_rect") {
		//console.log(is_rect);
        var items = this.id.split("_");
        var str1 = items.join("");
        items = this.id.split("_");
        var sendstr = "move " + str1
        board.socket.send(sendstr);
        remove_rect();
        console.log(sendstr);
        board.turn = 0;
      }
    }
  }
  else if (board.turn == 1 && board.tmp != null && this.firstElementChild == board.tmp) {
    //console.log("3", board.tmp.textContent);
    var s = String(this.id);
    var index = s.split('_');
    var tmp_x = index[0];
    var tmp_y = index[1];
    var ind_x = E_NUM_url.indexOf(tmp_x);
    var ind_y = S_NUM_url.indexOf(tmp_y);
    var text = board.tmp.textContent;
    if (this.firstElementChild.firstElementChild.className == "rect") {
      //console.log("4", "rect", board.tmp.context);
      if (this.childNodes.length > 0) {
        var b_children = this.childNodes;
        var is_rect = b_children[b_children.length - 1];
        if (is_rect.className == "rect") {
          /*while(b_children.length > 0)
          {
            is_rect = b_children[b_children.length-1];
            is_rect.remove();
          }*/
          //move
          //this.appendChild(board.tmp);
          var items = after_blank.id.split("_");
          var str1 = items.join("");
          items = this.id.split("_");
          var str2 = items.join("");
          var sendstr = "move " + str1;
          context.log("send " + sendstr);
          board.socket.send(sendstr);
          remove_rect();
          console.log(sendstr);
          board.turn = 0;
        }
        //board.tmp = null;
        //after_blank = null;
      }
    }
    after_blank = this;
  }
}
OCBoard.prototype.onclick_right = function (event) {
  document.oncontextmenu = function () { window.event.returnValue = false; }
  //console.log("onclick_right");
}


function remove_rect() {
  board.remove_div("blue_rect");
  board.remove_div("black_rect");
  board.remove_div("white_rect");
  board.remove_div("orange_rect");
}
function OS() {
  board.os_type = (navigator.platform.indexOf("Win", 0) != -1) ? '\r\n' : '\n';
}
function Save() {
  if (board.his_openf == "") { return; }
  var satext = "";
  var datacr = new Array();
  datacr = board.board_data.split('* ');
  //console.log(datacr);
  //"This is TEMPORARILY file. You can use it for DEBUG.";
  //20190721
  if (/\n/.test(datacr[0]) == false) {
    //satext += "This is TEMPORARILY file. You can use it for DEBUG." + board.os_type;
  }

  satext += "* " + datacr[1] + board.os_type;
  satext += "* " + datacr[2] + board.os_type;
  satext += "* " + datacr[3] + board.os_type;
  //console.log(satext);
  satext += "* " + datacr[4] + board.os_type;
  var i;
  for (i = 5; i <= 11; i++) {
    satext += "* " + datacr[i] + board.os_type;
  }
  //satext += board.os_type;

  satext += "* " + datacr[12] + board.os_type;
  satext += "* " + datacr[13] + board.os_type;
  //actlist
  console.log(board.actlist, board.his_step);
  var index = 1;
  var flag = false;
  for (i = 0; i <= board.his_step; i++) {
    var str1 = null;
    if ((i % 2) == 0) {
      if (i == 0) {
        str1 = "* " + index + ". ";
        flag = false;
      }
      else {
        str1 = board.os_type + "* " + index + ". ";
        flag = true;
      }
      satext += str1;
      index++;
    }
    satext += board.actlist[i].split(' ')[1];
    if ((i % 2) == 0) {
      satext += " ";
    }
  }
  if (flag || board.his_step == 0) {
    satext += board.os_type;
  }
  satext += "* Comment 0 0";
  satext += board.os_type;

  board.acttime[0] = board.acttime[0].replace(/\r?\n/g, "");
  satext += "0. " + board.acttime[0].split(board.split_str)[0].split(' ')[1] + board.os_type;
  index = 1;
  for (var i = 1; i < (board.his_step + 2); i++) {
    board.acttime[i] = board.acttime[i].replace(/\r?\n/g, "");
    satext += index + ". " + board.acttime[i].split(board.split_str)[0].split(' ')[1] + board.os_type;
    index++;
  }
  satext += "# #" + board.os_type;
  satext += "# #" + board.os_type;
  satext += "###" + board.os_type;
  var txtblob = new Blob([satext], { type: "text/plain;charset=utf-8" });
  var textarea = document.getElementById("filename").value;
  var filename = "";

  if (textarea != "") {
    filename = textarea + ".txt";
  }
  else {
    filename = "board.txt";
  }
  let url = window.URL.createObjectURL(txtblob);

  let downloadNode = document.createElement("a");
  downloadNode.style.display = "none";
  downloadNode.href = url;
  downloadNode.download = filename;

  document.body.appendChild(downloadNode);
  downloadNode.click();
  window.URL.revokeObjectURL(url);
}


console.log("js loading...");